@@ -1,26 +1,28 @@
-import sys
+from typing import Sequence
+from typing import Union
 
 import pytest
 from _pytest._code.code import ExceptionChainRepr
-from _pytest.pathlib import Path
+from _pytest._code.code import ExceptionRepr
+from _pytest.config import Config
+from _pytest.pytester import Pytester
 from _pytest.reports import CollectReport
 from _pytest.reports import TestReport
 
 
 class TestReportSerialization:
-    def test_xdist_longrepr_to_str_issue_241(self, testdir):
-        """
-        Regarding issue pytest-xdist#241
+    def test_xdist_longrepr_to_str_issue_241(self, pytester: Pytester) -> None:
+        """Regarding issue pytest-xdist#241.
 
         This test came originally from test_remote.py in xdist (ca03269).
         """
-        testdir.makepyfile(
+        pytester.makepyfile(
             """
             def test_a(): assert False
             def test_b(): pass
         """
         )
-        reprec = testdir.inline_run()
+        reprec = pytester.inline_run()
         reports = reprec.getreports("pytest_runtest_logreport")
         assert len(reports) == 6
         test_a_call = reports[1]
@@ -32,12 +34,12 @@ class TestReportSerialization:
         assert test_b_call.outcome == "passed"
         assert test_b_call._to_json()["longrepr"] is None
 
-    def test_xdist_report_longrepr_reprcrash_130(self, testdir) -> None:
+    def test_xdist_report_longrepr_reprcrash_130(self, pytester: Pytester) -> None:
         """Regarding issue pytest-xdist#130
 
         This test came originally from test_remote.py in xdist (ca03269).
         """
-        reprec = testdir.inline_runsource(
+        reprec = pytester.inline_runsource(
             """
                     def test_fail():
                         assert False, 'Expected Message'
@@ -47,15 +49,18 @@ class TestReportSerialization:
         assert len(reports) == 3
         rep = reports[1]
         added_section = ("Failure Metadata", "metadata metadata", "*")
+        assert isinstance(rep.longrepr, ExceptionRepr)
         rep.longrepr.sections.append(added_section)
         d = rep._to_json()
         a = TestReport._from_json(d)
-        assert a.longrepr is not None
+        assert isinstance(a.longrepr, ExceptionRepr)
         # Check assembled == rep
         assert a.__dict__.keys() == rep.__dict__.keys()
         for key in rep.__dict__.keys():
             if key != "longrepr":
                 assert getattr(a, key) == getattr(rep, key)
+        assert rep.longrepr.reprcrash is not None
+        assert a.longrepr.reprcrash is not None
         assert rep.longrepr.reprcrash.lineno == a.longrepr.reprcrash.lineno
         assert rep.longrepr.reprcrash.message == a.longrepr.reprcrash.message
         assert rep.longrepr.reprcrash.path == a.longrepr.reprcrash.path
@@ -68,14 +73,14 @@ class TestReportSerialization:
         # Missing section attribute PR171
         assert added_section in a.longrepr.sections
 
-    def test_reprentries_serialization_170(self, testdir) -> None:
+    def test_reprentries_serialization_170(self, pytester: Pytester) -> None:
         """Regarding issue pytest-xdist#170
 
         This test came originally from test_remote.py in xdist (ca03269).
         """
         from _pytest._code.code import ReprEntry
 
-        reprec = testdir.inline_runsource(
+        reprec = pytester.inline_runsource(
             """
                             def test_repr_entry():
                                 x = 0
@@ -86,32 +91,42 @@ class TestReportSerialization:
         reports = reprec.getreports("pytest_runtest_logreport")
         assert len(reports) == 3
         rep = reports[1]
+        assert isinstance(rep.longrepr, ExceptionRepr)
         d = rep._to_json()
         a = TestReport._from_json(d)
-        assert a.longrepr is not None
+        assert isinstance(a.longrepr, ExceptionRepr)
 
         rep_entries = rep.longrepr.reprtraceback.reprentries
         a_entries = a.longrepr.reprtraceback.reprentries
         for i in range(len(a_entries)):
-            assert isinstance(rep_entries[i], ReprEntry)
-            assert rep_entries[i].lines == a_entries[i].lines
-            assert rep_entries[i].reprfileloc.lineno == a_entries[i].reprfileloc.lineno
-            assert (
-                rep_entries[i].reprfileloc.message == a_entries[i].reprfileloc.message
-            )
-            assert rep_entries[i].reprfileloc.path == a_entries[i].reprfileloc.path
-            assert rep_entries[i].reprfuncargs.args == a_entries[i].reprfuncargs.args
-            assert rep_entries[i].reprlocals.lines == a_entries[i].reprlocals.lines
-            assert rep_entries[i].style == a_entries[i].style
+            rep_entry = rep_entries[i]
+            assert isinstance(rep_entry, ReprEntry)
+            assert rep_entry.reprfileloc is not None
+            assert rep_entry.reprfuncargs is not None
+            assert rep_entry.reprlocals is not None
 
-    def test_reprentries_serialization_196(self, testdir) -> None:
+            a_entry = a_entries[i]
+            assert isinstance(a_entry, ReprEntry)
+            assert a_entry.reprfileloc is not None
+            assert a_entry.reprfuncargs is not None
+            assert a_entry.reprlocals is not None
+
+            assert rep_entry.lines == a_entry.lines
+            assert rep_entry.reprfileloc.lineno == a_entry.reprfileloc.lineno
+            assert rep_entry.reprfileloc.message == a_entry.reprfileloc.message
+            assert rep_entry.reprfileloc.path == a_entry.reprfileloc.path
+            assert rep_entry.reprfuncargs.args == a_entry.reprfuncargs.args
+            assert rep_entry.reprlocals.lines == a_entry.reprlocals.lines
+            assert rep_entry.style == a_entry.style
+
+    def test_reprentries_serialization_196(self, pytester: Pytester) -> None:
         """Regarding issue pytest-xdist#196
 
         This test came originally from test_remote.py in xdist (ca03269).
         """
         from _pytest._code.code import ReprEntryNative
 
-        reprec = testdir.inline_runsource(
+        reprec = pytester.inline_runsource(
             """
                             def test_repr_entry_native():
                                 x = 0
@@ -122,9 +137,10 @@ class TestReportSerialization:
         reports = reprec.getreports("pytest_runtest_logreport")
         assert len(reports) == 3
         rep = reports[1]
+        assert isinstance(rep.longrepr, ExceptionRepr)
         d = rep._to_json()
         a = TestReport._from_json(d)
-        assert a.longrepr is not None
+        assert isinstance(a.longrepr, ExceptionRepr)
 
         rep_entries = rep.longrepr.reprtraceback.reprentries
         a_entries = a.longrepr.reprtraceback.reprentries
@@ -132,11 +148,9 @@ class TestReportSerialization:
             assert isinstance(rep_entries[i], ReprEntryNative)
             assert rep_entries[i].lines == a_entries[i].lines
 
-    def test_itemreport_outcomes(self, testdir):
-        """
-        This test came originally from test_remote.py in xdist (ca03269).
-        """
-        reprec = testdir.inline_runsource(
+    def test_itemreport_outcomes(self, pytester: Pytester) -> None:
+        # This test came originally from test_remote.py in xdist (ca03269).
+        reprec = pytester.inline_runsource(
             """
             import pytest
             def test_pass(): pass
@@ -160,7 +174,7 @@ class TestReportSerialization:
             assert newrep.failed == rep.failed
             assert newrep.skipped == rep.skipped
             if newrep.skipped and not hasattr(newrep, "wasxfail"):
-                assert newrep.longrepr is not None
+                assert isinstance(newrep.longrepr, tuple)
                 assert len(newrep.longrepr) == 3
             assert newrep.outcome == rep.outcome
             assert newrep.when == rep.when
@@ -168,9 +182,9 @@ class TestReportSerialization:
             if rep.failed:
                 assert newrep.longreprtext == rep.longreprtext
 
-    def test_collectreport_passed(self, testdir):
+    def test_collectreport_passed(self, pytester: Pytester) -> None:
         """This test came originally from test_remote.py in xdist (ca03269)."""
-        reprec = testdir.inline_runsource("def test_func(): pass")
+        reprec = pytester.inline_runsource("def test_func(): pass")
         reports = reprec.getreports("pytest_collectreport")
         for rep in reports:
             d = rep._to_json()
@@ -179,9 +193,9 @@ class TestReportSerialization:
             assert newrep.failed == rep.failed
             assert newrep.skipped == rep.skipped
 
-    def test_collectreport_fail(self, testdir):
+    def test_collectreport_fail(self, pytester: Pytester) -> None:
         """This test came originally from test_remote.py in xdist (ca03269)."""
-        reprec = testdir.inline_runsource("qwe abc")
+        reprec = pytester.inline_runsource("qwe abc")
         reports = reprec.getreports("pytest_collectreport")
         assert reports
         for rep in reports:
@@ -193,13 +207,13 @@ class TestReportSerialization:
             if rep.failed:
                 assert newrep.longrepr == str(rep.longrepr)
 
-    def test_extended_report_deserialization(self, testdir):
+    def test_extended_report_deserialization(self, pytester: Pytester) -> None:
         """This test came originally from test_remote.py in xdist (ca03269)."""
-        reprec = testdir.inline_runsource("qwe abc")
+        reprec = pytester.inline_runsource("qwe abc")
         reports = reprec.getreports("pytest_collectreport")
         assert reports
         for rep in reports:
-            rep.extra = True
+            rep.extra = True  # type: ignore[attr-defined]
             d = rep._to_json()
             newrep = CollectReport._from_json(d)
             assert newrep.extra
@@ -209,33 +223,41 @@ class TestReportSerialization:
             if rep.failed:
                 assert newrep.longrepr == str(rep.longrepr)
 
-    def test_paths_support(self, testdir):
-        """Report attributes which are py.path or pathlib objects should become strings."""
-        testdir.makepyfile(
+    def test_paths_support(self, pytester: Pytester) -> None:
+        """Report attributes which are path-like should become strings."""
+        pytester.makepyfile(
             """
             def test_a():
                 assert False
         """
         )
-        reprec = testdir.inline_run()
+
+        class MyPathLike:
+            def __init__(self, path: str) -> None:
+                self.path = path
+
+            def __fspath__(self) -> str:
+                return self.path
+
+        reprec = pytester.inline_run()
         reports = reprec.getreports("pytest_runtest_logreport")
         assert len(reports) == 3
         test_a_call = reports[1]
-        test_a_call.path1 = testdir.tmpdir
-        test_a_call.path2 = Path(testdir.tmpdir)
+        test_a_call.path1 = MyPathLike(str(pytester.path))  # type: ignore[attr-defined]
+        test_a_call.path2 = pytester.path  # type: ignore[attr-defined]
         data = test_a_call._to_json()
-        assert data["path1"] == str(testdir.tmpdir)
-        assert data["path2"] == str(testdir.tmpdir)
+        assert data["path1"] == str(pytester.path)
+        assert data["path2"] == str(pytester.path)
 
-    def test_deserialization_failure(self, testdir):
+    def test_deserialization_failure(self, pytester: Pytester) -> None:
         """Check handling of failure during deserialization of report types."""
-        testdir.makepyfile(
+        pytester.makepyfile(
             """
             def test_a():
                 assert False
         """
         )
-        reprec = testdir.inline_run()
+        reprec = pytester.inline_run()
         reports = reprec.getreports("pytest_runtest_logreport")
         assert len(reports) == 3
         test_a_call = reports[1]
@@ -250,9 +272,11 @@ class TestReportSerialization:
             TestReport._from_json(data)
 
     @pytest.mark.parametrize("report_class", [TestReport, CollectReport])
-    def test_chained_exceptions(self, testdir, tw_mock, report_class):
+    def test_chained_exceptions(
+        self, pytester: Pytester, tw_mock, report_class
+    ) -> None:
         """Check serialization/deserialization of report objects containing chained exceptions (#5786)"""
-        testdir.makepyfile(
+        pytester.makepyfile(
             """
             def foo():
                 raise ValueError('value error')
@@ -268,9 +292,11 @@ class TestReportSerialization:
             )
         )
 
-        reprec = testdir.inline_run()
+        reprec = pytester.inline_run()
         if report_class is TestReport:
-            reports = reprec.getreports("pytest_runtest_logreport")
+            reports: Union[
+                Sequence[TestReport], Sequence[CollectReport]
+            ] = reprec.getreports("pytest_runtest_logreport")
             # we have 3 reports: setup/call/teardown
             assert len(reports) == 3
             # get the call report
@@ -282,7 +308,7 @@ class TestReportSerialization:
             assert len(reports) == 2
             report = reports[1]
 
-        def check_longrepr(longrepr):
+        def check_longrepr(longrepr: ExceptionChainRepr) -> None:
             """Check the attributes of the given longrepr object according to the test file.
 
             We can get away with testing both CollectReport and TestReport with this function because
@@ -306,6 +332,7 @@ class TestReportSerialization:
 
         assert report.failed
         assert len(report.sections) == 0
+        assert isinstance(report.longrepr, ExceptionChainRepr)
         report.longrepr.addsection("title", "contents", "=")
         check_longrepr(report.longrepr)
 
@@ -320,58 +347,32 @@ class TestReportSerialization:
         # elsewhere and we do check the contents of the longrepr object after loading it.
         loaded_report.longrepr.toterminal(tw_mock)
 
-    def test_chained_exceptions_no_reprcrash(self, testdir, tw_mock) -> None:
+    def test_chained_exceptions_no_reprcrash(self, pytester: Pytester, tw_mock) -> None:
         """Regression test for tracebacks without a reprcrash (#5971)
 
         This happens notably on exceptions raised by multiprocess.pool: the exception transfer
         from subprocess to main process creates an artificial exception, which ExceptionInfo
         can't obtain the ReprFileLocation from.
         """
-        # somehow in Python 3.5 on Windows this test fails with:
-        #   File "c:\...\3.5.4\x64\Lib\multiprocessing\connection.py", line 302, in _recv_bytes
-        #     overlapped=True)
-        # OSError: [WinError 6] The handle is invalid
-        #
-        # so in this platform we opted to use a mock traceback which is identical to the
-        # one produced by the multiprocessing module
-        if sys.version_info[:2] <= (3, 5) and sys.platform.startswith("win"):
-            testdir.makepyfile(
-                """
-                # equivalent of multiprocessing.pool.RemoteTraceback
-                class RemoteTraceback(Exception):
-                    def __init__(self, tb):
-                        self.tb = tb
-                    def __str__(self):
-                        return self.tb
-                def test_a():
-                    try:
-                        raise ValueError('value error')
-                    except ValueError as e:
-                        # equivalent to how multiprocessing.pool.rebuild_exc does it
-                        e.__cause__ = RemoteTraceback('runtime error')
-                        raise e
+        pytester.makepyfile(
             """
-            )
-        else:
-            testdir.makepyfile(
-                """
-                from concurrent.futures import ProcessPoolExecutor
+            from concurrent.futures import ProcessPoolExecutor
 
-                def func():
-                    raise ValueError('value error')
+            def func():
+                raise ValueError('value error')
 
-                def test_a():
-                    with ProcessPoolExecutor() as p:
-                        p.submit(func).result()
-            """
-            )
+            def test_a():
+                with ProcessPoolExecutor() as p:
+                    p.submit(func).result()
+        """
+        )
 
-        testdir.syspathinsert()
-        reprec = testdir.inline_run()
+        pytester.syspathinsert()
+        reprec = pytester.inline_run()
 
         reports = reprec.getreports("pytest_runtest_logreport")
 
-        def check_longrepr(longrepr) -> None:
+        def check_longrepr(longrepr: object) -> None:
             assert isinstance(longrepr, ExceptionChainRepr)
             assert len(longrepr.chain) == 2
             entry1, entry2 = longrepr.chain
@@ -400,13 +401,17 @@ class TestReportSerialization:
 
         # for same reasons as previous test, ensure we don't blow up here
         assert loaded_report.longrepr is not None
+        assert isinstance(loaded_report.longrepr, ExceptionChainRepr)
         loaded_report.longrepr.toterminal(tw_mock)
 
-    def test_report_prevent_ConftestImportFailure_hiding_exception(self, testdir):
-        sub_dir = testdir.tmpdir.join("ns").ensure_dir()
-        sub_dir.join("conftest").new(ext=".py").write("import unknown")
+    def test_report_prevent_ConftestImportFailure_hiding_exception(
+        self, pytester: Pytester
+    ) -> None:
+        sub_dir = pytester.path.joinpath("ns")
+        sub_dir.mkdir()
+        sub_dir.joinpath("conftest.py").write_text("import unknown")
 
-        result = testdir.runpytest_subprocess(".")
+        result = pytester.runpytest_subprocess(".")
         result.stdout.fnmatch_lines(["E   *Error: No module named 'unknown'"])
         result.stdout.no_fnmatch_line("ERROR  - *ConftestImportFailure*")
 
@@ -414,14 +419,14 @@ class TestReportSerialization:
 class TestHooks:
     """Test that the hooks are working correctly for plugins"""
 
-    def test_test_report(self, testdir, pytestconfig):
-        testdir.makepyfile(
+    def test_test_report(self, pytester: Pytester, pytestconfig: Config) -> None:
+        pytester.makepyfile(
             """
             def test_a(): assert False
             def test_b(): pass
         """
         )
-        reprec = testdir.inline_run()
+        reprec = pytester.inline_run()
         reports = reprec.getreports("pytest_runtest_logreport")
         assert len(reports) == 6
         for rep in reports:
@@ -436,14 +441,14 @@ class TestHooks:
             assert new_rep.when == rep.when
             assert new_rep.outcome == rep.outcome
 
-    def test_collect_report(self, testdir, pytestconfig):
-        testdir.makepyfile(
+    def test_collect_report(self, pytester: Pytester, pytestconfig: Config) -> None:
+        pytester.makepyfile(
             """
             def test_a(): assert False
             def test_b(): pass
         """
         )
-        reprec = testdir.inline_run()
+        reprec = pytester.inline_run()
         reports = reprec.getreports("pytest_collectreport")
         assert len(reports) == 2
         for rep in reports:
@@ -461,13 +466,15 @@ class TestHooks:
     @pytest.mark.parametrize(
         "hook_name", ["pytest_runtest_logreport", "pytest_collectreport"]
     )
-    def test_invalid_report_types(self, testdir, pytestconfig, hook_name):
-        testdir.makepyfile(
+    def test_invalid_report_types(
+        self, pytester: Pytester, pytestconfig: Config, hook_name: str
+    ) -> None:
+        pytester.makepyfile(
             """
             def test_a(): pass
             """
         )
-        reprec = testdir.inline_run()
+        reprec = pytester.inline_run()
         reports = reprec.getreports(hook_name)
         assert reports
         rep = reports[0]
