@@ -1,18 +1,19 @@
 import inspect
 
-import _pytest.warning_types
 import pytest
+from _pytest import warning_types
+from _pytest.pytester import Pytester
 
 
 @pytest.mark.parametrize(
     "warning_class",
     [
         w
-        for n, w in vars(_pytest.warning_types).items()
+        for n, w in vars(warning_types).items()
         if inspect.isclass(w) and issubclass(w, Warning)
     ],
 )
-def test_warning_types(warning_class):
+def test_warning_types(warning_class: UserWarning) -> None:
     """Make sure all warnings declared in _pytest.warning_types are displayed as coming
     from 'pytest' instead of the internal module (#5452).
     """
@@ -20,11 +21,11 @@ def test_warning_types(warning_class):
 
 
 @pytest.mark.filterwarnings("error::pytest.PytestWarning")
-def test_pytest_warnings_repr_integration_test(testdir):
+def test_pytest_warnings_repr_integration_test(pytester: Pytester) -> None:
     """Small integration test to ensure our small hack of setting the __module__ attribute
     of our warnings actually works (#5452).
     """
-    testdir.makepyfile(
+    pytester.makepyfile(
         """
         import pytest
         import warnings
@@ -33,5 +34,13 @@ def test_pytest_warnings_repr_integration_test(testdir):
             warnings.warn(pytest.PytestWarning("some warning"))
     """
     )
-    result = testdir.runpytest()
+    result = pytester.runpytest()
     result.stdout.fnmatch_lines(["E       pytest.PytestWarning: some warning"])
+
+
+@pytest.mark.filterwarnings("error")
+def test_warn_explicit_for_annotates_errors_with_location():
+    with pytest.raises(Warning, match="(?m)test\n at .*python_api.py:\\d+"):
+        warning_types.warn_explicit_for(
+            pytest.raises, warning_types.PytestWarning("test")  # type: ignore
+        )
